import {store} from '@redux/store';
import {Platform} from 'react-native';

import getUserAgent from './userAgent';

import {
  AccountName,
  Password,
  XAuthorization,
  Username,
  UserId,
} from '../configs/types';
import MyUserManager from '@user/MyUserManager';

export const apiPath = {
  backend: {
    prefix: 'https://api-partyapp.ex.umbach.dev/api/v1',
  },
};

export enum apiBackendRequest {
  LOGIN = '/user/login',
  APP_START = '/user',
  GET_USER_PROFILE = '/users/:userId',
  LOGOUT = '/user/logout',
  SIGN_UP = '/user/signup',
  CHECK_ACCOUNT_NAME = '/user/check/:accountName',
  /*REGISTER_STEP_1 = '/admin/users/email',
  REGISTER_RESEND_MAIL = '/admin/users/email/resend',
  REGISTER_STEP_2 = '/verify/email/:xToken/:verifyId',
  REGISTER_STEP_FINAL = '/admin/users',
  REGISTER_STEP_FINAL_ACCOUNT_NAME_CHECK = '/admin/users/validation/accountname',
  LOGIN = '/user', */
}

type requestGET = {[key: string]: string};
type requestHeader = {[key: string]: string | undefined};

export interface defaultRequest {
  status?: 200 | 400 | 422 | 500;
  error?: string;
  requestGET?: requestGET;
  requestHeader?: requestHeader;
}

/*
interface REGISTER_STEP_1 extends defaultRequest {
  path: apiBackendRequest.REGISTER_STEP_1;

  request: {
    Email: EMail;
  };
  response: {
    XToken: XToken | undefined;
  };
}
interface REGISTER_RESEND_MAIL extends defaultRequest {
  path: apiBackendRequest.REGISTER_RESEND_MAIL;
  requestHeader: {
    'X-Token': XToken | undefined;
  };
  request: {
    Email: EMail;
  };
  response: {
    XToken: XToken | undefined;
  };
}

interface REGISTER_STEP_2 extends defaultRequest {
  path: apiBackendRequest.REGISTER_STEP_2;

  requestGET: {
    ':xToken': XToken;
    ':verifyId': verifyId;
  };
}

interface REGISTER_STEP_FINAL_ACCOUNT_NAME_CHECK extends defaultRequest {
  path: apiBackendRequest.REGISTER_STEP_FINAL_ACCOUNT_NAME_CHECK;

  request: {
    AccountName: AccountName;
  };
  // response: {
  //   XToken: XToken | undefined;
  //};
}

interface REGISTER_STEP_FINAL extends defaultRequest {
  path: apiBackendRequest.REGISTER_STEP_FINAL;
  requestHeader: {
    'X-Token': XToken;
  };
  request: {
    AccountName: AccountName;
    Username: Username;
    Password: Password;
  };
  response?: {
    XAuthorization: XAuthorization;
    UserId: UserId;
    WebSocketSessionId: WebSocketSessionId;
  };
}
 */
interface LOGIN extends defaultRequest {
  path: apiBackendRequest.LOGIN;
  request: {
    AccountName: AccountName;
    //Email: EMail;
    Password: Password;
  };
  response?: {
    XAuthorization: XAuthorization;
    UserId: UserId;
    Username: Username;
  };
}

interface SIGN_UP extends defaultRequest {
  path: apiBackendRequest.SIGN_UP;
  request: {
    AccountName: AccountName;
    Username: Username;
    Password: Password;
  };
  response?: {
    XAuthorization: XAuthorization;
    UserId: UserId;
    Username: Username;
  };
}

interface GET_USER_PROFILE extends defaultRequest {
  path: apiBackendRequest.GET_USER_PROFILE;

  requestGET: {
    ':userId': UserId;
  };
  response: {
    //AccountName: AccountName;
    Username: Username;
    Description: string;
    FollowersCount: number;
    FollowingCount: number;
    XpLevel: number;
    XpPoints: number;
    AvatarUrl: string;
  };
}

interface CHECK_ACCOUNT_NAME extends defaultRequest {
  path: apiBackendRequest.CHECK_ACCOUNT_NAME;

  requestGET: {
    ':accountName': AccountName;
  };
  response: {};
}

interface APP_START extends defaultRequest {
  path: apiBackendRequest.APP_START;

  response: {
    accountName: AccountName;
    username: Username;
    /* TokenValid: boolean;
    AccountName: AccountName;
    Username: Username;
    Description: string;
    FollowersCount: number;
    FollowingCount: number;
    XpLevel: number;
    XpPoints: number;
    AccountStatus: number;
    AvatarUrl: string;
    Events: any; */
  };
}

interface LOGOUT extends defaultRequest {
  path: apiBackendRequest.LOGOUT;
}

type FetchTypes =
  | LOGIN
  | SIGN_UP
  /*| REGISTER_STEP_1
  | REGISTER_RESEND_MAIL
  | REGISTER_STEP_2
  | REGISTER_STEP_FINAL
  | REGISTER_STEP_FINAL_ACCOUNT_NAME_CHECK
  | LOGIN*/
  | GET_USER_PROFILE
  | APP_START
  | LOGOUT
  | CHECK_ACCOUNT_NAME;
/*
function isA(obj: any): obj is REGISTER_STEP_1 {
  return obj.request !== undefined;
} */

export function makeRequest<T1 extends FetchTypes>(type: T1): Promise<T1> {
  let makeRequestObj: any = type;

  let path = type.path.toString();

  if (type.requestGET !== undefined) {
    for (let key in type.requestGET) {
      path = path.replaceAll(key, (type.requestGET as requestGET)[key]);
    }
  }

  // Simple POST request with a JSON body using fetch

  let headers: HeadersInit_ = {
    'Content-Type': 'application/json',
    'X-App-Version': store
      .getState()
      .nonSaveVariables.currentAppVersion.toString(),
    'X-Language': store.getState().appVariables.lang.details.langCode,
  };

  if (type.requestHeader !== undefined) {
    for (let key in type.requestHeader) {
      let val = (type.requestHeader as requestHeader)[key];

      if (val !== undefined) headers[key] = val;
    }
  }

  if (Platform.OS === 'android' || Platform.OS === 'ios')
    headers['User-Agent'] = getUserAgent();

  const SessionId = MyUserManager.getSessionId();
  if (SessionId !== undefined) headers['X-Authorization'] = SessionId;

  console.log('session id: ' + SessionId);

  const requestOptions: RequestInit = {
    method: makeRequestObj.request !== undefined ? 'POST' : 'GET',

    headers,
    body:
      makeRequestObj.request !== undefined
        ? JSON.stringify(makeRequestObj.request)
        : undefined,
  };

  return new Promise<T1>((resolve, reject) => {
    fetch(apiPath.backend.prefix + path, requestOptions)
      .then(response => {
        makeRequestObj.status = response.status;

        if (makeRequestObj.response === undefined) return {};

        return response.json();
      })
      .then(data => {
        makeRequestObj.response = data;

        //console.log(makeRequestObj);
        if (makeRequestObj.status !== 200) reject(makeRequestObj);
        else resolve(makeRequestObj);
      })
      .catch(error => {
        //console.error(error, makeRequestObj);
        makeRequestObj.error = error;
        reject(makeRequestObj);
      });
  });
}
