import {
  createNativeStackNavigator,
  NativeStackNavigationProp,
} from '@react-navigation/native-stack';
import {useSelector} from 'react-redux';
import {RootState} from '@redux/store';
import {MyIconButton} from '@components/MyButton';
import {useNavigation} from '@react-navigation/native';
import {ProfileOverview} from '@pages/profile/profile';
import {lazy} from 'react';
import {MySuspenseFallback} from '@components/MySuspenseFallback';

const LazyProfileSettings = lazy(() =>
  import('@pages/profile/profile').then(module => ({
    default: module.ProfileSettings,
  })),
);

const LazyUpdateUsername = lazy(() =>
  import('@pages/profile/profile').then(module => ({
    default: module.UpdateUsername,
  })),
);

const LazyUpdatePassword = lazy(() =>
  import('@pages/profile/profile').then(module => ({
    default: module.UpdatePassword,
  })),
);

const LazyHelp = lazy(() =>
  import('@pages/profile/help').then(module => ({default: module.Help})),
);

const LazyAbout = lazy(() =>
  import('@pages/profile/about').then(module => ({default: module.About})),
);

const Help = () => {
  return (
    <MySuspenseFallback>
      <LazyHelp />
    </MySuspenseFallback>
  );
};

const About = () => {
  return (
    <MySuspenseFallback>
      <LazyAbout />
    </MySuspenseFallback>
  );
};

const ProfileSettings = () => {
  return (
    <MySuspenseFallback>
      <LazyProfileSettings />
    </MySuspenseFallback>
  );
};

const UpdateUsername = () => {
  return (
    <MySuspenseFallback>
      <LazyUpdateUsername />
    </MySuspenseFallback>
  );
};

const UpdatePassword = () => {
  return (
    <MySuspenseFallback>
      <LazyUpdatePassword />
    </MySuspenseFallback>
  );
};

export const ProfileTabName = 'Profile';

export type ProfileStackNavigatorParamList = {
  Overview: undefined;
  Settings: undefined;
  UpdateUsername: undefined;
  UpdatePhoneNumber: undefined;
  VerifyUpdatedPhoneNumber: undefined;
  UpdatePassword: undefined;
  Help: undefined;
  About: undefined;
};

const ProfileStack =
  createNativeStackNavigator<ProfileStackNavigatorParamList>();

export type ProfileScreenNavigationProp =
  NativeStackNavigationProp<ProfileStackNavigatorParamList>;

export default function ProfileTab() {
  const navigation = useNavigation<ProfileScreenNavigationProp>();
  const lang = useSelector(
    (state: RootState) => state.appVariables.lang.navigation.home.profile,
  );
  const currentTheme = useSelector(
    (state: RootState) => state.nonSaveVariables.theme.colors,
  );

  const headerStyle = {
    backgroundColor: currentTheme.backgroundDark400,
  };

  return (
    <ProfileStack.Navigator>
      <ProfileStack.Screen
        name="Overview"
        options={{
          animation: 'slide_from_left',
          title: lang.overview,
          headerShown: true,
          headerStyle: headerStyle,
          headerShadowVisible: false,
          headerRight: () => (
            <MyIconButton
              MyIconProps={{name: 'settings', size: 24}}
              onPress={() => navigation.navigate('Settings')}
            />
          ),
        }}
        component={ProfileOverview}
      />

      <ProfileStack.Screen
        name="Settings"
        options={{
          animation: 'slide_from_right',
          title: lang.settings,
          headerShown: true,
          headerStyle: headerStyle,
          headerShadowVisible: false,
          headerTitleAlign: 'center',
        }}
        component={ProfileSettings}
      />

      <ProfileStack.Screen
        name="UpdateUsername"
        options={{
          animation: 'slide_from_right',
          title: lang.updateUsername,
          headerShown: true,
          headerStyle: headerStyle,
          headerShadowVisible: false,
          headerTitleAlign: 'center',
        }}
        component={UpdateUsername}
      />

      <ProfileStack.Screen
        name="UpdatePassword"
        options={{
          animation: 'slide_from_right',
          title: lang.updatePassword,
          headerShown: true,
          headerStyle: headerStyle,
          headerShadowVisible: false,
          headerTitleAlign: 'center',
        }}
        component={UpdatePassword}
      />

      <ProfileStack.Screen
        name="Help"
        options={{
          animation: 'slide_from_right',
          title: lang.help,
          headerShown: true,
          headerStyle: headerStyle,
          headerShadowVisible: false,
          headerTitleAlign: 'center',
        }}
        component={Help}
      />

      <ProfileStack.Screen
        name="About"
        options={{
          animation: 'slide_from_right',
          title: lang.about,
          headerShown: true,
          headerStyle: headerStyle,
          headerShadowVisible: false,
          headerTitleAlign: 'center',
        }}
        component={About}
      />
    </ProfileStack.Navigator>
  );
}
