import {MyTouchableOpacity} from '@components/MyTouchableOpacity';
import {Spinner, Text} from '@gluestack-ui/themed';
import {RootState} from '@redux/store';
import {useState} from 'react';
import {Image} from 'react-native';
import {FlatList} from 'react-native';
import {View} from 'react-native';
import {useSelector} from 'react-redux';

const baseUrls = [
  'https://www.w3schools.com/w3css/img_lights.jpg',
  'https://suburbanmen.com/wp-content/uploads/2021/12/instagram-crush-emilia-bte-20211202-125.jpg',
  'https://i.ytimg.com/vi/X0I6PbIWmPo/hqdefault.jpg',
  'https://i.ytimg.com/vi/RDrJ1VmRi0w/hqdefault.jpg',
];

interface Event {
  id: number;
  url: string;
  title: string;
  description: string;
}

/*
const events = [
  {
    id: 1,
    url: 'https://www.w3schools.com/w3css/img_lights.jpg',
    title: 'Bootshaus',
    description: '01.12.2023 20:00 Uhr',
  },
  {
    id: 2,
    url: 'https://suburbanmen.com/wp-content/uploads/2021/12/instagram-crush-emilia-bte-20211202-125.jpg',
    title: 'Bootshaus',
    description: '01.12.2023 20:00 Uhr',
  },
  {
    id: 3,
    url: 'https://i.ytimg.com/vi/X0I6PbIWmPo/hqdefault.jpg',
    title: 'Bootshaus',
    description: '01.12.2023 20:00 Uhr',
  },
  {
    id: 4,
    url: 'https://i.ytimg.com/vi/RDrJ1VmRi0w/hqdefault.jpg',
    title: 'Bootshaus',
    description: '01.12.2023 20:00 Uhr',
  },
];*/

export function CalendarScreen() {
  const currentTheme = useSelector(
    (state: RootState) => state.nonSaveVariables.theme.colors,
  );

  const [refreshing, setRefreshing] = useState(false);
  //const [paginationPage, setPaginationPage] = useState(1);

  const newEvents = () => {
    let list = [];

    for (let i = 1; i <= 10; i++) {
      const randomUrlIndex = Math.floor(Math.random() * baseUrls.length);

      list.push({
        id: i,
        url: baseUrls[randomUrlIndex],
        title: 'Bootshaus',
        description: '01.12.2023 20:00 Uhr',
      });
    }

    return list;
  };

  const [events, setEvents] = useState(newEvents());

  return (
    <View>
      <FlatList
        data={events}
        keyExtractor={(item, index) => `${item.id}${index}`}
        renderItem={({item}) => (
          <MyTouchableOpacity
            key={item.id}
            style={{
              margin: 12,
              padding: 12,
              borderRadius: 10,
              backgroundColor: currentTheme.backgroundDark300,
              marginBottom: 12,
              flexDirection: 'row',
              gap: 12,
            }}>
            <View style={{backgroundColor: currentTheme.backgroundDark200}}>
              <Image
                style={{
                  width: 100,
                  height: 60,
                }}
                source={{uri: item.url}}
              />
            </View>

            <View>
              <Text size="xl" bold>
                {item.title}
              </Text>
              <Text>{item.description}</Text>
            </View>
          </MyTouchableOpacity>
        )}
        refreshing={refreshing}
        onRefresh={() => {
          console.log('refresh');

          setRefreshing(true);

          setTimeout(() => {
            setRefreshing(false);
          }, 1000);
        }}
        onEndReached={() => {
          console.log('end reached');

          setEvents([...events, ...newEvents()]);
        }}
        onEndReachedThreshold={0}
        ListFooterComponent={() => {
          return (
            <View
              style={{
                height: 40,
                justifyContent: 'center',
                alignItems: 'center',
              }}>
              <Spinner />
            </View>
          );
        }}
      />
    </View>
  );
}
