import {MapStackNavigatorParamList} from '@navigation/tabs/main/MapTab';
import {
  createNativeStackNavigator,
  NativeStackNavigationProp,
} from '@react-navigation/native-stack';
import {RootState} from '@redux/store';
import {View, Text} from 'react-native';
import {useSelector} from 'react-redux';
import EventPage from './EventPage';

import Overview from './CreateEvent/Overview';

export type EventCreationNavigatorParamList = {
  Overview: undefined;
  PublicEvent: undefined;
  PrivateEvent: undefined;
};

const EventCreationStack =
  createNativeStackNavigator<MapStackNavigatorParamList>();

export type EventCreationScreenNavigationProp =
  NativeStackNavigationProp<MapStackNavigatorParamList>;

function CreateEventPage() {
  const lang = useSelector((state: RootState) => state.appVariables.lang);
  const currentTheme = useSelector(
    (state: RootState) => state.nonSaveVariables.theme.colors,
  );

  const headerStyle = {
    backgroundColor: currentTheme.backgroundDark400,
  };

  return (
    <EventCreationStack.Navigator initialRouteName="Overview">
      <EventCreationStack.Screen
        name="Overview"
        options={{
          animation: 'simple_push',
          title: lang.eventCreateion.newEventTitle,
          headerShown: true,
          headerStyle: headerStyle,
          headerShadowVisible: false,
          headerTitleAlign: 'center',
        }}
        component={Overview}
      />
    </EventCreationStack.Navigator>
  );
}

export default CreateEventPage;
