import {StyleSheet, View, Text, Image, ViewStyle} from 'react-native';
import Mapbox, {MarkerView} from '@rnmapbox/maps';

import React, {useState} from 'react'; // Add useState import

import DisplayMarkerList from '@components/map/DisplayMarkerList';

import getLocationData from '@components/map/cluster/getData';
import {PA_Point} from '@components/map/types';
import {RootState, store} from '@redux/store';
import {appNonSaveVarActions} from '@configs/appNonSaveVarReducer';
import {Position} from '@rnmapbox/maps/src/types/Position';

import {Dimensions, Pressable} from 'react-native';
import {MyIconButton} from '@components/MyButton';
import {MyTouchableOpacity} from '@components/MyTouchableOpacity';
import {MyIcon} from '@components/MyIcon';
import {useSelector} from 'react-redux';
import {Button, ButtonIcon} from '@gluestack-ui/themed';
import LinearGradient from 'react-native-linear-gradient';

import MaterialIcon from 'react-native-vector-icons/MaterialIcons';
import {MapScreenNavigationProp} from '@navigation/tabs/main/MapTab';
import {useNavigation} from '@react-navigation/native';

Mapbox.setAccessToken(
  'pk.eyJ1IjoidGl0YW5pdW1iYWNoIiwiYSI6ImNscGgzZGJxMDAwbHQyaXA2N3BtOWUxbWkifQ.x-f8JJxwQHWmPFI3P6Qn-w',
);

let lastCameraChange = 0;
let isRerenderData = 0;

const IconButtonSize = 28;
const IconButtonCircleSize = IconButtonSize * 1.75;

const IconMarginRight = 15;

export const Map = () => {
  const mapRef = React.useRef<Mapbox.MapView | null>(null);
  //const [mapMarkers, setMapMarkers] = useState<PA_Point[]>([]); // Add useState for visibleBounds
  const [lastDataRerender, setLastDataRerender] = useState(0); // Add useState for visibleBounds

  const colors = useSelector(
    (state: RootState) => state.nonSaveVariables.theme.colors,
  );

  const navigation = useNavigation<MapScreenNavigationProp>();

  const getVisibleBounds = async () => {
    // return when lastDataRerender is 300ms ago
    const now = Date.now();

    if (now - lastDataRerender < 2000) return;

    isRerenderData = now;

    //console.log('----1');

    if (mapRef.current) {
      const visibleBounds = await (
        mapRef.current as Mapbox.MapView
      ).getVisibleBounds();

      /*  const deviceWidth = Dimensions.get('window').width;
      const deviceHeight = Dimensions.get('window').height;

      const visibleBounds: [Position, Position] = [
        await mapRef.current.getCoordinateFromView([100, 100]),
        await mapRef.current.getCoordinateFromView([100, 200]),
      ];

      //log visibleBounds
      console.log(visibleBounds);
      */

      //const zoomLevel = await (mapRef.current as Mapbox.MapView).getZoom();

      //console.log('----2', Date.now() - now + 'ms');

      //console.log(visibleBounds);
      let markerData = (await getLocationData(visibleBounds, 0))?.locations;

      //console.log('----3', Date.now() - now + 'ms');

      store.dispatch(appNonSaveVarActions.setMapMarkers(markerData || []));
      //setMapMarkers(markerData || []);

      //console.log('----4', Date.now() - now + 'ms');
    }

    isRerenderData = 0;
  };

  function onCameraChanged() {
    const now = Date.now();

    if (now - lastCameraChange > 500) {
      lastCameraChange = now;
      setTimeout(getVisibleBounds, 500);
      //console.log('------------5', Date.now() - now + 'ms');
    }
  }

  function onMapMoveFinished() {
    //console.log('xxxxxxxxxxxxxxxxxxxdad');

    (async () => {
      await getVisibleBounds();
    })();
  }

  React.useEffect(() => {
    let intervalId: NodeJS.Timeout;

    if (mapRef.current !== null) {
    }

    const executeInterval = async () => {
      await getVisibleBounds();
      intervalId = setTimeout(executeInterval, 50000);
    };

    executeInterval();

    return () => {
      clearInterval(intervalId);
    };
  }, []);

  return (
    <View style={styles.page}>
      <View style={styles.container}>
        <Mapbox.MapView
          ref={mapRef}
          onMapIdle={onMapMoveFinished}
          onCameraChanged={onCameraChanged}
          style={styles.map}
          compassEnabled={true}
          scaleBarEnabled={false}
          //attributionPosition={{bottom: 75, left: 100}}
          //logoPosition={{bottom: 75, left: 10}}
          attributionPosition={{top: 10, left: 100}}
          logoPosition={{top: 10, left: 10}}
          styleURL="mapbox://styles/titaniumbach/clpij5uoo00o301pg2dj23j0m"
          projection="globe"
          compassPosition={{top: 10, right: IconMarginRight}}
          compassImage={'mycompassimagekey'}>
          <DisplayMarkerList />
        </Mapbox.MapView>
        <MapIconButton
          MyIconProps={{
            name: 'group',
            size: IconButtonSize,
            backgroundColor: colors.backgroundDark300,
          }}
          onPress={() => {}}
          style={{
            top: 70,
            right: IconMarginRight,
          }}
        />

        <MapIconButton
          MyIconProps={{
            name: 'tune',
            size: IconButtonSize,
            backgroundColor: colors.backgroundDark300,
          }}
          onPress={() => {}}
          style={{
            top: 70 + IconButtonCircleSize + 10,
            right: IconMarginRight,
          }}
        />

        <MapIconButton
          MyIconProps={{
            name: 'add',
            size: IconButtonSize,
            color: colors.backgroundLight300,
          }}
          onPress={() => {
            navigation.navigate('CreateEvent', {screen: 'Overview'});
          }}
          style={{
            bottom: 90 + 10 + IconButtonCircleSize,
            right: IconMarginRight,
          }}
          type="primary"
        />
        <MapIconButton
          MyIconProps={{
            name: 'search',
            size: IconButtonSize,
            backgroundColor: colors.backgroundDark300,
          }}
          onPress={() => {}}
          style={{
            bottom: 90,
            right: IconMarginRight,
          }}
        />
      </View>
    </View>
  );
};

const styles = StyleSheet.create({
  page: {
    flex: 1,
    justifyContent: 'center',
    alignItems: 'center',
  },
  container: {
    height: '100%',
    width: '100%',
  },
  map: {
    flex: 1,
  },
});

function MapIconButton({
  onPress,
  MyIconProps,
  style,
  type,
}: {
  onPress: () => void;
  MyIconProps: {
    name: string;
    color?: string;
    size: number;
    backgroundColor?: string;
  };
  style?: ViewStyle;
  type?: 'primary' | 'secondary';
}) {
  const currentTheme = useSelector(
    (state: RootState) => state.nonSaveVariables.theme,
  );

  const viewStyle: ViewStyle = {
    position: 'absolute',

    width: IconButtonCircleSize,
    height: IconButtonCircleSize,
    borderRadius: MyIconProps.size, // This will make the View circular
    justifyContent: 'center', // Center the icon vertically
    alignItems: 'center', // Center the icon horizontally
    elevation: 5,
    overflow: 'hidden',
    ...style,
  };

  const pressableStyle: ViewStyle = {
    width: '100%',
    height: '100%',
    justifyContent: 'center', // Center the icon vertically
    alignItems: 'center', // Center the icon horizontally
  };

  if (type === 'primary') {
    return (
      <LinearGradient
        colors={[
          currentTheme.colors.secondary200,
          currentTheme.colors.primary400,
        ]}
        start={{x: 0, y: 1}}
        end={{x: 1, y: 0}}
        style={{elevation: 50, ...viewStyle}}>
        <Pressable
          android_ripple={{color: '#3e00a8'}}
          style={pressableStyle}
          onPress={onPress}>
          <MaterialIcon
            name={MyIconProps.name}
            size={MyIconProps.size}
            color={MyIconProps.color}
          />
        </Pressable>
      </LinearGradient>
    );
  }

  return (
    <View
      style={{
        backgroundColor: MyIconProps.backgroundColor, // Set the background color
        ...viewStyle,
      }}>
      <Pressable
        android_ripple={{color: '#fff2', foreground: false}}
        style={pressableStyle}
        onPress={onPress}>
        <MaterialIcon
          name={MyIconProps.name}
          size={MyIconProps.size}
          color={MyIconProps.color}
        />
      </Pressable>
    </View>
  );
}
