import {MyButton} from '@components/MyButton';
import {
  AccountNameAvailable,
  MyAccountNameInput,
  MyPasswordInput,
  MyUsernameInput,
  accountNameValid,
  passwordValid,
  usernameValid,
} from '@components/MyCommonInputs';
import {MyScreenContainer} from '@components/MyScreenContainer';
import {MyTitle} from '@components/MyTitle';
import showToast from '@components/MyToast';
import {appVarActions} from '@configs/appVarReducer';
import {passwordOptions} from '@configs/types';
import {useToast} from '@gluestack-ui/themed';
import {ToBase64} from '@helper/base64';
import {apiBackendRequest, makeRequest} from '@helper/request';
import {RootScreenNavigationProp} from '@navigation/navigation';
import {
  ContentContainer,
  navigateToHome,
} from '@navigation/registration/registration';
import {useNavigation} from '@react-navigation/native';
import {RootState, store} from '@redux/store';
import MyUserManager from '@user/MyUserManager';
import {useState} from 'react';
import {Text} from 'react-native';
import {View} from 'react-native';
import {useSelector} from 'react-redux';
import reactStringReplace from 'react-string-replace';

function Title({text, description}: {text: string; description?: string}) {
  return (
    <>
      <MyTitle text={text} textAlign="left" paddingBottom={0} />
      {description && <Text>{description}</Text>}
    </>
  );
}

export function SignUpStepUsername() {
  const lang = useSelector(
    (state: RootState) => state.appVariables.lang.registration,
  );
  const navigation = useNavigation<RootScreenNavigationProp>();

  const registerProcess = useSelector(
    (state: RootState) => state.appVariables.preferences.RegisterProcess,
  );

  const [username, setUsername] = useState('');

  return (
    <MyScreenContainer
      style={{
        flexDirection: 'column',
      }}>
      <ContentContainer>
        <Title
          text={lang.signUpStepUsername.title}
          description={lang.signUpStepUsername.description}
        />

        <View style={{gap: 12, marginTop: 20}}>
          <MyUsernameInput
            username={username}
            setUsername={value => setUsername(value)}
          />

          <MyButton
            type="secondary"
            text={lang.buttonNext}
            style={{marginBottom: 20}}
            disabled={!usernameValid(username)}
            onPress={() => {
              let rp = {...registerProcess};

              rp.Username = username;

              store.dispatch(appVarActions.setRegisterProcess(rp));

              navigation.navigate('Registration', {
                screen: /*'SignUpStepPhoneNumber' */ 'SignUpStepPassword',
              });
            }}
          />
        </View>
      </ContentContainer>
    </MyScreenContainer>
  );
}
/*
export function SignUpStepPhoneNumber() {
  const lang = useSelector(
    (state: RootState) => state.appVariables.lang.registration,
  );
  const navigation = useNavigation<RootScreenNavigationProp>();

  return (
    <MyScreenContainer
      style={{
        flexDirection: 'column',
      }}>
      <ContentContainer>
        <Title text={lang.signUpStepPhoneNumber.title} />

        <View style={{gap: 12, marginTop: 20}}>
          <MyIconInput
            text={lang.signUpStepPhoneNumber.inputPhoneNumber}
            iconName="phone-iphone"
            keyboardType="phone-pad"
          />

          <MyButton
            type="secondary"
            text={lang.buttonNext}
            style={{marginBottom: 20}}
            onPress={() => {
              navigation.navigate('Registration', {
                screen: 'SignUpStepVerifyPhoneNumber',
              });
            }}
          />
        </View>
      </ContentContainer>
    </MyScreenContainer>
  );
}

export function SignUpStepVerifyPhoneNumber() {
  const lang = useSelector(
    (state: RootState) => state.appVariables.lang.registration,
  );
  const currentTheme = useSelector(
    (state: RootState) => state.nonSaveVariables.theme.colors,
  );
  const navigation = useNavigation<RootScreenNavigationProp>();

  return (
    <MyScreenContainer
      style={{
        flexDirection: 'column',
      }}>
      <ContentContainer>
        <Title
          text={lang.signUpStepVerifyPhoneNumber.title}
          description={`${lang.signUpStepVerifyPhoneNumber.description} +49 15** ******43`}
        />

        <View style={{gap: 12, marginTop: 20}}>
          <ConfirmationCodeInput />

          <MyButton
            type="secondary"
            text={lang.buttonNext}
            style={{marginBottom: 2}}
            onPress={() => {
              navigation.navigate('Registration', {
                screen: 'SignUpStepPassword',
              });
            }}
          />
          <View style={{alignItems: 'center'}}>
            <MyClickableText
              text={lang.signUpStepVerifyPhoneNumber.clickableTextResendCode}
              color={currentTheme.textLight400}
            />
          </View>
        </View>
      </ContentContainer>
    </MyScreenContainer>
  );
}
*/
export function SignUpStepPassword() {
  const lang = useSelector(
    (state: RootState) => state.appVariables.lang.registration,
  );
  const navigation = useNavigation<RootScreenNavigationProp>();

  const registerProcess = useSelector(
    (state: RootState) => state.appVariables.preferences.RegisterProcess,
  );

  const [password, setPassword] = useState('');

  const descriptionText = reactStringReplace(
    lang.signUpStepPassword.description,
    '${minLength}',
    () => {
      return passwordOptions.minLength.toString();
    },
  );

  return (
    <MyScreenContainer
      style={{
        flexDirection: 'column',
      }}>
      <ContentContainer>
        <Title
          text={lang.signUpStepPassword.title}
          description={descriptionText.join('')}
        />

        <View style={{gap: 12, marginTop: 20}}>
          <MyPasswordInput
            password={password}
            setPassword={value => setPassword(value)}
          />

          <MyButton
            type="secondary"
            text={lang.buttonNext}
            style={{marginBottom: 2}}
            disabled={!passwordValid(password)}
            onPress={() => {
              let rp = {...registerProcess};

              rp.Password = ToBase64(password);

              store.dispatch(appVarActions.setRegisterProcess(rp));

              navigation.navigate('Registration', {
                screen: 'SignUpStepAccountName',
              });
            }}
          />
        </View>
      </ContentContainer>
    </MyScreenContainer>
  );
}

export function SignUpStepAccountName() {
  const lang = useSelector(
    (state: RootState) => state.appVariables.lang.registration,
  );

  const navigation = useNavigation<RootScreenNavigationProp>();
  const toast = useToast();

  const registerProcess = useSelector(
    (state: RootState) => state.appVariables.preferences.RegisterProcess,
  );

  const [isLoading, setIsLoading] = useState(false);
  const [accountName, setAccountName] = useState('');

  const [isAccountNameAvailable, setIsAccountNameAvailable] = useState(
    AccountNameAvailable.Loading,
  );

  return (
    <MyScreenContainer
      scrollView
      style={{
        flexDirection: 'column',
      }}>
      <ContentContainer>
        <Title
          text={lang.signUpStepAccountName.title}
          description={lang.signUpStepAccountName.description}
        />

        <View style={{gap: 12, marginTop: 20}}>
          <MyAccountNameInput
            accountName={accountName}
            setAccountName={value => setAccountName(value)}
            isAccountNameAvailable={isAccountNameAvailable}
            setIsAccountNameAvailable={value =>
              setIsAccountNameAvailable(value)
            }
            checkAccountNameAvailability
          />

          <MyButton
            type="primary"
            text={lang.signUpStepAccountName.buttonGetStarted}
            style={{marginBottom: 2}}
            isLoading={isLoading}
            disabled={
              !accountNameValid(accountName) ||
              isAccountNameAvailable !== AccountNameAvailable.Available
            }
            onPress={() => {
              setIsLoading(true);

              let rp = {...registerProcess};

              rp.AccountName = accountName;

              store.dispatch(appVarActions.setRegisterProcess(rp));

              makeRequest({
                path: apiBackendRequest.SIGN_UP,
                request: rp,
                method: 'POST',
                response: {
                  XAuthorization: '',
                  UserId: '',
                  Username: '',
                },
              })
                .then(resp => {
                  MyUserManager.createNewMyUser(
                    resp.response.UserId,
                    accountName,
                    resp.response.Username,
                    resp.response.XAuthorization,
                  )
                    .then(async () => {
                      setIsLoading(false);
                      navigateToHome(navigation);
                    })
                    .catch(err => {
                      console.log('catch', err);
                    });
                })
                .catch(reason => {
                  console.log('error', reason);

                  setIsLoading(false);

                  let text =
                    store.getState().appVariables.lang.registration
                      .signUpStepAccountName.signUpError[
                      reason.status as number
                    ];

                  console.log('text', text, reason.status);

                  showToast(toast, {
                    title: 'Failed',
                    variant: 'solid',
                    action: 'error',
                    description: text,
                    isClosable: true,
                    //rest: {colorScheme: 'primary'},
                  });
                });
            }}
          />
        </View>
      </ContentContainer>
    </MyScreenContainer>
  );
}
